'use strict';

// tides: https://tidesandcurrents.noaa.gov/map/index.html

// marine forecast: https://marine.weather.gov/MapClick.php?lon=-123.11327&lat=38.36586

// weather: https://api.weather.gov/points/38.3659,-123.1133

const fs = require('fs');
const fsp = fs.promises;
const fetch = require('node-fetch');
const xml2js = require('xml2js');
const moment = require('moment');

let { sources } = JSON.parse(fs.readFileSync('../sources.json'));

function getMarineForecast(lat, lon) {
    return fetch(`https://marine.weather.gov/MapClick.php?lat=${lat}&lon=${lon}&unit=0&lg=english&FcstType=json`).then(res => res.json());
}

function getMarineAndWriteToFile(lat, lon, file) {
    getMarineForecast(lat, lon)
        .then(data => fs.writeFile(file,
                                   JSON.stringify(data),
                                   x => console.log(x)));
}

function extractMarineForecast(obj) {
    const { time, data } = obj;
    return time.startValidTime.map((t, i) => {
        const sentences = data.text[i].split(/\. /);
        let swellDetails = sentences[2].match(/\b[NESW]+ [a-z]*[ ]*\d+ t?o?f?t? ?\d*/g);
        if (swellDetails === null) {
            swellDetails = sentences[3].match(/\b[NESW]+ [a-z]*[ ]*\d+ t?o?f?t? ?\d*/g);
        }
        if (swellDetails !== null) {
            swellDetails = swellDetails
                .map(x => {
                    return {
                        direction: x.match(/\b[NESW]+/g)[0],
                        height: Math.max.apply(Math, x.match(/\d+/g).map(z => parseInt(z)))
                    };
                });
        } else {
            swellDetails = [];
        }
        const windHeight = Math.max.apply(Math, sentences[3].match(/\d+/g).map(z => parseInt(z)));
        return {
            date: new Date(t),
            text: data.text[i],
            swellText: sentences[2],
            windText: sentences[3],
            swellDetails,
            windHeight
        };
    });
}

function storeMarineForecast(db, location, objs) {
    return objs
        .map(x => {
            return db.update({
                type: 'marine',
                location,
                date: x.date
            }, {
                type: 'marine',
                location,
                ...x
            }, { upsert: true });
        });
}

function convertXMLTide(file) {
    const parser = new xml2js.Parser();
    return parser.parseStringPromise(
        fs.readFileSync(file + '.xml'))
        .then(result => fs.writeFileSync(file + '.json',
                                         JSON.stringify(result)));
    // fs.readFileSync(file + '.xml', function(err, data) {
    //     parser.parseString(data, function (err, result) {
    //         fs.writeFile(file + '.json',
    //                      JSON.stringify(result),
    //                      x => console.log(x));
    //     });
    // });
}

function extractTideData(input) {
    return input.datainfo.data[0].item
        .map(item => {
            return {
                date: item.date[0],
                time: item.time[0],
                low: item.highlow[0] === 'L',
                distance: parseFloat(item.pred_in_ft[0])
            };
        });
}

function extractWeatherData(input) {
    return input.properties.periods;
}

function storeWeatherForecast(db, location, objs) {
    return objs
        .map(x => {
            return db.update({
                type: 'weather',
                location,
                date: new Date(x.startTime)
            }, {
                type: 'weather',
                location,
                ...x,
                date: new Date(x.startTime),
                startTime: new Date(x.startTime),
                endTime: new Date(x.endTime)
            }, { upsert: true });
        });
}

function getWeatherForecast(id) {
    return fetch(`https://api.weather.gov/gridpoints/MTR/${id}/forecast`)
        .then(res => res.json());
}

function getWeatherAndWriteToFile(id, file) {
    getWeatherForecast(id)
        .then(data => fs.writeFile(file,
                                   JSON.stringify(data),
                                   x => console.log(x)));
}

function loadExtractWriteTideData(fileIn, fileOut) {
    fs.writeFileSync(fileOut,
                     JSON.stringify({ data: extractTideData(JSON.parse(fs.readFileSync(fileIn))) }));
}

function loadExtractStoreTideData(db, location, fileIn) {
    return extractTideData(JSON.parse(fs.readFileSync(fileIn)))
        .map(x => {
            const d = moment(x.date + ' ' + x.time + '+0000', 'YYYY-MM-DD LT Z').toDate();
            return db.update({
                type: 'tide',
                location,
                date: d
            }, {
                type: 'tide',
                location,
                low: x.low,
                date: d,
                distance: x.distance
            }, { upsert: true });
        });
}

function loadExtractWriteMarineForecast(fileIn, fileOut) {
    fs.writeFileSync(fileOut,
                     JSON.stringify({ data: extractMarineForecast(JSON.parse(fs.readFileSync(fileIn))) }));
}

function loadExtractWriteWeatherData(fileIn, fileOut) {
    fs.writeFileSync(fileOut,
                     JSON.stringify({ data: extractWeatherData(JSON.parse(fs.readFileSync(fileIn))) }));
}

function processSource(db, source) {
    sources = JSON.parse(fs.readFileSync('../sources.json')).sources;
    console.log('processing ' + source.name);
    return db.findOne({ location: source.id, type: 'tide' })
        .then(data => {
            if (data === null) {
                return convertXMLTide(`routes/data/${source.id}-tide-raw`);
            }
        })
        .then(() => db.findOne({ location: source.id, type: 'tide' }))
        .then(data => {
            if (data === null) {
                loadExtractWriteTideData(`routes/data/${source.id}-tide-raw.json`,
                                         `routes/data/${source.id}-tide.json`);
                return Promise.all(loadExtractStoreTideData(db, source.id, `routes/data/${source.id}-tide-raw.json`));
            }
        })
        .then(() => getMarineForecast(source.marine.lat, source.marine.lon))
        .then(data => storeMarineForecast(db, source.id, extractMarineForecast(data)))
        .then(() => getWeatherForecast(source.weather))
        .then(data => storeWeatherForecast(db, source.id, extractWeatherData(data)))
        .then(() => console.log('done processing ' + source.name));
}

exports.processSources = function(db) {
    return sources.map(s => processSource(db, s));
}

// processSource(sources['sonoma']);

// getMarineAndWriteToFile('37.4695', '-122.521', 'pillar-point-marine-forecast-raw.json');
// https://tidesandcurrents.noaa.gov/cgi-bin/predictiondownload.cgi?&stnid=9414131&threshold=&thresholdDirection=greaterThan&bdate=2020&timezone=LST/LDT&datum=MLLW&clock=12hour&type=xml&annual=true
// convertXMLTide('/data/pillar-point-tide-raw');
// loadExtractWriteTideData('pillar-point-tide-raw.json', 'pillar-point-tide.json');
// loadExtractWriteMarineForecast('pillar-point-marine-forecast-raw.json',
//                                'pillar-point-marine-forecast.json');
// getWeatherAndWriteToFile('81,113', 'pillar-point-weather-raw.json');
// loadExtractWriteWeatherData('pillar-point-weather-raw.json',
//                            'pillar-point-weather.json');

// getMarineAndWriteToFile('38.36586', '-123.11327', 'sonoma-marine-forecast-raw.json');
// loadExtractWriteMarineForecast('sonoma-marine-forecast-raw.json',
//                                'sonoma-marine-forecast.json');
// convertXMLTide('/sonoma-tide-raw');
// loadExtractWriteTideData('sonoma-tide-raw.json', 'sonoma-tide.json');
// getWeatherAndWriteToFile('69,157', 'sonoma-weather-raw.json');
// loadExtractWriteWeatherData('sonoma-weather-raw.json',
//                            'sonoma-weather.json');
